@@ -1,6 +1,5 @@
-from platform import release
+from typing import Optional, Literal
 from .StandardConstants import *
-from typing import Optional
 from lxml import etree
 import datetime
 
@@ -26,7 +25,7 @@ class TranslateableTag:
         """Sets the translated text"""
         self._translations[lang] = text
 
-    def get_aviable_languages(self) -> list[str]:
+    def get_available_languages(self) -> list[str]:
         """Returns a list with all languages of the tag"""
         return list(self._translations.keys())
 
@@ -34,9 +33,9 @@ class TranslateableTag:
         """Load a list of Tags"""
         for i in tag_list:
             if i.get("{http://www.w3.org/XML/1998/namespace}lang") is None:
-                self._text = i.text
+                self._text = i.text.strip()
             else:
-                self._translations[i.get("{http://www.w3.org/XML/1998/namespace}lang")] = i.text
+                self._translations[i.get("{http://www.w3.org/XML/1998/namespace}lang")] = i.text.strip()
 
     def write_tags(self, parent_tag: etree.Element, tag_name: str) -> None:
         """Writes a Tag"""
@@ -71,16 +70,16 @@ class TranslateableList:
         if tag.get("{http://www.w3.org/XML/1998/namespace}lang") is None:
             for i in tag.getchildren():
                 if i.get("{http://www.w3.org/XML/1998/namespace}lang") is None:
-                    self.default_list.append(i.text)
+                    self.default_list.append(i.text.strip())
                 else:
                     if i.get("{http://www.w3.org/XML/1998/namespace}lang") not in self.translated_lists:
                         self.translated_lists[i.get("{http://www.w3.org/XML/1998/namespace}lang")] = []
-                    self.translated_lists[i.get("{http://www.w3.org/XML/1998/namespace}lang")].append(i.text)
+                    self.translated_lists[i.get("{http://www.w3.org/XML/1998/namespace}lang")].append(i.text.strip())
         else:
             if tag.get("{http://www.w3.org/XML/1998/namespace}lang") not in self.translated_lists:
                 self.translated_lists[tag.get("{http://www.w3.org/XML/1998/namespace}lang")] = []
             for i in tag.getchildren():
-                self.translated_lists[tag.get("{http://www.w3.org/XML/1998/namespace}lang")].append(i.text)
+                self.translated_lists[tag.get("{http://www.w3.org/XML/1998/namespace}lang")].append(i.text.strip())
 
     def write_all_tag(self, parent_tag: etree.Element, tag_name: str) -> None:
         for i in range(len(self.default_list)):
@@ -176,7 +175,7 @@ class Description:
                         self.items.append(paragraph_item)
                     paragraph_list.clear()
                     paragraph_list.append(i)
-            else:
+            elif i.tag in ("ul", "ol"):
                 if len(paragraph_list) != 0:
                     paragraph_item = DescriptionParagraph()
                     paragraph_item.load_tags(paragraph_list)
@@ -246,38 +245,62 @@ class Release:
 
 class Image:
     def __init__(self) -> None:
-        self.url: str
+        self.url: str = ""
         "The image URL"
 
-        self.type: str
+        self.type: Literal["source", "thumbnail"] = "source"
         "The image type"
 
+        self.width: Optional[int] = None
+        "The width"
+
+        self.height: Optional[int] = None
+        "The height"
+
+        self.language: Optional[str] = None
+        "The language"
+
     def load_tag(self, tag: etree.Element):
         "Loads a image tag"
-        self.url = tag.text
-        self.type = tag.get("type")
+        self.url = tag.text.strip()
+        self.type = tag.get("type", "source")
+
+        try:
+            self.width = int(tag.get("width"))
+        except (ValueError, TypeError):
+            self.width = None
+
+        try:
+            self.height = int(tag.get("height"))
+        except (ValueError, TypeError):
+            self.height = None
+
+        self.language = tag.get("{http://www.w3.org/XML/1998/namespace}lang")
 
 
 class Screenshot:
     def __init__(self) -> None:
-        self.source_image: Image
-        "The source image"
-
-        self.thumbnail_images: list[Image] = []
+        self.images: list[Image] = []
         "the list with thumbnail images"
 
         self.caption: TranslateableTag = TranslateableTag()
         "The caption"
+    
+    def get_source_image(self) -> Optional[Image]:
+        for image in self.images:
+            if image.type in ("source", None):
+                return image
+        return None
+
+    def get_thumbnail_images(self) -> list[Image]:
+        return [image for image in self.images if image.type == "thumbnail"]
 
     def load_tag(self, tag: etree.Element):
         "Load a screenshot tag"
         for i in tag.findall("image"):
             img = Image()
             img.load_tag(i)
-            if img.type == "source":
-                self.source_image = img
-            elif img.type == "thumbnail":
-                self.thumbnail_images.append(img)
+            self.images.append(img)
 
         self.caption.load_tags(tag.findall("caption"))
 
@@ -383,9 +406,9 @@ class AppstreamComponent:
         for i in CONTROL_TYPES:
             self.controls[i] = None
 
-    def get_aviable_languages(self) -> list[str]:
-        "Returns a list with all aviable languages of the Component"
-        lang_list = self.name.get_aviable_languages() + self.summary.get_aviable_languages() + self.developer_name.get_aviable_languages()
+    def get_available_languages(self) -> list[str]:
+        "Returns a list with all available languages of the Component"
+        lang_list = self.name.get_available_languages() + self.summary.get_available_languages() + self.developer_name.get_available_languages()
         return list(set(lang_list))
 
     def _parse_relation_tag(self, tag: etree.Element) -> None:
@@ -393,11 +416,11 @@ class AppstreamComponent:
         relation = tag.tag
 
         for i in tag.findall("control"):
-            if i.text in CONTROL_TYPES:
-                self.controls[i.text] = relation
+            if i.text.strip() in CONTROL_TYPES:
+                self.controls[i.text.strip()] = relation
 
     def parse_component_tag(self, tag: etree._ElementTree) -> None:
-        self.id = tag.find("id").text
+        self.id = tag.find("id").text.strip()
 
         try:
             self.type = tag.xpath("/component")[0].get("type")
@@ -416,43 +439,43 @@ class AppstreamComponent:
 
         metadata_license_tag = tag.find("metadata_license")
         if metadata_license_tag is not None:
-            self.metadata_license = metadata_license_tag.text
+            self.metadata_license = metadata_license_tag.text.strip()
 
         project_license_tag = tag.find("project_license")
         if project_license_tag is not None:
-            self.project_license = project_license_tag.text
+            self.project_license = project_license_tag.text.strip()
 
         categories_tag = tag.find("categories")
         if categories_tag is not None:
             for i in categories_tag.findall("category"):
-                self.categories.append(i.text)
+                self.categories.append(i.text.strip())
 
         for i in tag.findall("url"):
             if i.get("type") in URL_TYPES:
-                self.urls[i.get("type")] = i.text
+                self.urls[i.get("type")] = i.text.strip()
 
         oars_tag = tag.find("content_rating")
         if oars_tag is not None:
             for i in oars_tag.findall("content_attribute"):
-                if i.get("id") in OARS_ATTRIBUTE_TYPES and i.text in OARS_VALUE_TYPES:
-                    self.oars[i.get("id")] = i.text
+                if i.get("id") in OARS_ATTRIBUTE_TYPES and i.text.strip() in OARS_VALUE_TYPES:
+                    self.oars[i.get("id")] = i.text.strip()
 
         categories_tag = tag.find("categories")
         if categories_tag is not None:
             for i in categories_tag.findall("category"):
-                self.categories.append(i.text)
+                self.categories.append(i.text.strip())
 
         provides_tag = tag.find("provides")
         if provides_tag is not None:
             for i in provides_tag.getchildren():
                 if i.tag in PROVIDES_TYPES:
-                    self.provides[i.tag].append(i.text)
+                    self.provides[i.tag].append(i.text.strip())
 
         # For backwards compatibility. See: https://www.freedesktop.org/software/appstream/docs/chap-Metadata.html#tag-mimetypes
         mimetypes_tag = tag.find("mimetypes")
         if mimetypes_tag is not None:
             for i in mimetypes_tag.findall("mimetype"):
-                self.provides["mediatype"].append(i.text)
+                self.provides["mediatype"].append(i.text.strip())
 
         releases_tag = tag.find("releases")
         if releases_tag is not None:
@@ -470,19 +493,19 @@ class AppstreamComponent:
 
         project_group_tag = tag.find("project_group")
         if project_group_tag is not None:
-            self.project_group = project_group_tag.text
+            self.project_group = project_group_tag.text.strip()
 
         for i in tag.findall("translation"):
             trans_dict = {}
             trans_dict["type"] = i.get("type")
-            trans_dict["value"] = i.text
+            trans_dict["value"] = i.text.strip()
             self.translation.append(trans_dict)
 
         languages_tag = tag.find("languages")
         if languages_tag is not None:
             for i in languages_tag.findall("lang"):
                 try:
-                    self.languages[i.text] = int(i.get("percentage") or 100)
+                    self.languages[i.text.strip()] = int(i.get("percentage") or 100)
                 except ValueError:
                     pass
 
@@ -504,20 +527,29 @@ class AppstreamComponent:
         kudos_tag = tag.find("kudos")
         if kudos_tag is not None:
             for i in kudos_tag.findall("kudo"):
-                self.kudos.append(i.text)
+                self.kudos.append(i.text.strip())
 
         update_contact_tag = tag.find("update_contact")
         if update_contact_tag is not None:
-            self.update_contact = update_contact_tag.text
+            self.update_contact = update_contact_tag.text.strip()
 
         for i in tag.findall("extends"):
-            self.extends.append(i.text)
+            self.extends.append(i.text.strip())
 
     def load_file(self, path: str) -> None:
-        """Load a appdata.xml or metainfo.xml file"""
+        """Load an appdata.xml or metainfo.xml file"""
         root = etree.parse(path)
         self.parse_component_tag(root)
 
+    def load_bytes(self, data: bytes, encoding: Optional[str] = None) -> None:
+        """Load an appdata.xml or metainfo.xml byte string"""
+        root = etree.fromstring(data, parser=etree.XMLParser(encoding=encoding))
+        self.parse_component_tag(root)
+
+    def load_string(self, text: str) -> None:
+        """Load an appdata.xml or metainfo.xml string"""
+        self.load_bytes(text.encode("utf-8"), encoding="utf-8")
+
     def _get_relation_tag(self, parent_tag: etree.Element, relation: str) -> None:
         relation_tag = etree.SubElement(parent_tag, relation)
 
